package examples.keyedfile;

/**
 * IBM Confidential                                                  
 * OCO Source Materials                                              
 * Satin Data Integration Toolkit
 * Prog License number                                               
 * (C) Copyright IBM Corp. 2002                                      
 * The source code for this program is not published or              
 * otherwise divested of its trade secrets, irrespective of          
 * what has been deposited with the US Copyright Office.             
 */

import org.apache.commons.logging.*;

import com.ibm.mqe.*;

import com.ibm.OS4690.*;

import com.ibm.retail.di.service.util.*;
import com.ibm.retail.di.service.mom.mqe.*;

import examples.queuemanager.*;

/**
 * Service to perform keyed file lookups.
 * @author Dan Streetman
 */
public class MqeKeyedFileService extends MqeListenerService
{
	/**
	 * Initialize.
	 * <p>
	 * This calls the superclass's method, then opens the keyed file
	 * specified in the Context, and checks the key length and record size.
	 * The keyed file is opened in shared read-only mode.
	 * @exception Exception If an error occurs during initialization.
	 */
	protected void doInit() throws Exception
	{
		super.doInit();
                mqeI = MqeInterfaceFactory.createMqeInterface(getContext());

		String keyedFileName = NamingTools.lookup(getContext(), KEYEDFILE_PROPERTY, true);

		keyedFile = new KeyedFile(keyedFileName, "r", KeyedFile.SHARED_READ_ACCESS);
		keyLength = keyedFile.getKeyLength();
		recordSize = keyedFile.getRecordSize();
	}

	/**
	 * Process a request for a keyed-file lookup.
	 * <p>
	 * The message <i>must</i> have these fields:
	 * <ul>
	 * <li>(ascii)RemoteQueueManager - This is the name of the remote Queue Manager.</li>
	 * <li>(ascii)RemoteQueue - This is the name of the remote Queue.</li>
	 * <li>(byte[])Data - This is the key to use for the lookup.</li>
	 * </ul>
	 * The local MQe QueueManager must already have a remote Queue set up that points
	 * to the specified remote QM and Queue.  The key must be the correct length,
	 * as a keyed file uses fixed length key, specified at keyed file creation.
	 * @param message The message to process.
	 * @exception Exception If an error occurs during processing.
	 */
	protected void onMessage(MQeMsgObject message) throws Exception
	{
		logger.debug("Got MQeMsgObject message");

		String remoteQueueManager = message.getAscii("RemoteQueueManager");
		String remoteQueue = message.getAscii("RemoteQueue");

		byte[] record = new byte[recordSize];
		byte[] key = message.getArrayOfByte("Data");

		if (key.length != keyLength)
			throw new Exception("Requested key length " + key.length + " invalid, KeyedFile key length is " + keyLength);

		System.arraycopy(key, 0, record, 0, keyLength);

		int length = 0;

		try {
			length = keyedFile.read(record, KeyedFile.NO_LOCK);
		} catch ( Exception e ) {
			System.out.println("Exception while reading KeyedFile : " + e.toString());
			return;
		}

		logger.debug("Read " + length + " bytes from KeyedFile.");
		logger.debug("Record : " + new String(record));

		MQeMsgObject reply = new MQeMsgObject();
		reply.putArrayOfByte("Data", record);

		mqeI.getQueueManager().putMessage(remoteQueueManager, remoteQueue, reply, null, 0);
	}

	private KeyedFile keyedFile = null;
	private int keyLength = -1;
	private int recordSize = -1;
        private MqeInterface mqeI = null;

	static Log logger = LogFactory.getLog(MqeKeyedFileService.class);

	public static final String KEYEDFILE_PROPERTY = "MqeKeyedFileService.keyedfile";
}
