package examples.testing;

import java.io.*;
import java.net.*;

import com.ibm.retail.di.net.protocol.*;
import com.ibm.retail.di.net.protocol.soep.SoepURLConnection;
import com.ibm.retail.di.util.ByteTools;
import com.ibm.retail.di.xml.messaging.soap.SoapUtil;


/**
 * Sends a test message to a DIF service. This example demonstrates using
 * the Java URL class to communicate with DIF. 
 * <p>
 * The target DIF service must respond to socket or pipe connections. Choose
 * the <code>soep</code> protocol for 4690 pipes-based communications or the 
 * <code>soeps</code> protocol for sockets-based communications.
 * </p>
 * <p>
 * To easily set up this example, enable either the SoepClientService or
 * SoepsClientService within the DIF runtime by including one of the 
 * following lines in difuser.pro:<br>
 * <ul>
 * <li><code>container.services=SoepClientService</code></li>
 * <li><code>container.services=SoepsClientService</code></li>
 * </ul>
 * <p><b>Note: You can only have one definition for <code>container.services</code>,
 * so be sure to comment out (#) any existing definition.</b>
 * </p>
 * Next, in difuser.pro, specify the EchoActor as the listener of the
 * service. The service will pass inbound messages to the EchoActor, which
 * will simply respond with the same content as the input. To associate
 * the EchoActor with the service, add one of these lines to difuser.pro:<br>
 * <ul>
 * <li><code>SoepClientService.listener=EchoActor</code></li>
 * <li><code>SoepsClientService.listener=EchoActor</code></li>
 * </uL>
 * <br>
 * Finally, define the EchoActor in difuser.pro:<br>
 * <code>EchoActor.uri=local:examples.testing.EchoActor</code>
 * </p>
 * 
 * @author C. Capps
 */
public class SendNonSoapMessage
{
    
    public static final String DEFAULT_SOEP_URL = "soep://pi_server/pi_tool";
    
    public static final String DEFAULT_SOEPS_URL = "soeps://127.0.0.1:6696";

    public static void main(String[] args)
        throws Exception
    {

        /* establish the URL of the DIF service to connect to */
        String urlStr = DEFAULT_SOEPS_URL;
        /* create a Java URL reference for this address */
        URL url = new URL (urlStr);

        /* open a connection to the address for synchronous request/response */
        URLConnection uc = url.openConnection();
        
        /* if we used SOEP, typecast the URL connection so we can use additional functions */
        SoepURLConnection difUrlConnection = (SoepURLConnection)uc; 

        /* SoepsURLConnection difUrlConnection = (SoepsURLConnection)uc; */
                                                                             
        /* property/value pairs can be passed along with the message body; any
           number of these can be set */
                                                                
        /* some properties have special meaning: these are defined by the
         * MessageHeaders class; the most important is the "message profile Id",
         * which typically must be set on every message and determines how
         * the receiving service should process the message
         */    
        difUrlConnection.setRequestProperty(                                  
            MessageHeaders.getMessageProfileIdFieldName(),
            "TLOG");
        
        /* determine the message body: this can be any binary data, but here
           we use a String for simplicity */        
        String message = "Hello, world! Hello, world! Hello, world! Hello, world! Hello, world! Hello, world! Hello, world!";
        byte[] buffer = message.getBytes();

        /* write the body to the OutputStream given by the URLConnection */            
        OutputStream out = difUrlConnection.getOutputStream(); 
        out.write (buffer);
        out.flush();
            
        /* now wait for a response from the server; this blocks until the underlying
         * protocol times out; for pipes, there is never a timeout, for sockets it
         * depends on the SO_TIMEOUT property from dif.pro.
         */
        InputStream in = difUrlConnection.getInputStream();
        
        /* retrieve the response headers (property/value pairs from the server) */
        MessageHeaders headers = difUrlConnection.getResponseHeaders();

        /* the "fault count" is a special header property that denotes an error in 
         * processing the message on the server; a value greater than 0 is a failure
         */
        if (SoapUtil.getHeaderFaultCount (headers) != 0) {

            String faultMsg = headers.getHeaderField (MessageHeaders.getFaultMessageFieldName());
            
            /* if there's a fault, print the associated message */
            String printableFaultMsg = faultMsg;
            if (printableFaultMsg.length() > 60) {
                printableFaultMsg = faultMsg.substring (0, 60) + "..."; // up to 60 chars
            }
            
            System.err.println ("Failed: " + printableFaultMsg);
        }
        
        else {
            
            /* use the ByteTools convenience method to fully read the content of the
             * input stream into a buffer
             */             
            buffer = ByteTools.getStreamAsBytes (in);
            
            /* convert the buffer to a string */
            String response = new String (buffer);
            
            System.out.println ("Message sent successfully.");
            System.out.println ("\nHeaders:");
            difUrlConnection.getResponseHeaders().list (System.out);
                         
            System.out.println ("\nPayload:\n" + response);
        }
        
        /* close underlying pipe or sockets */
        difUrlConnection.disconnect();        
        
        /* the URL can be reused to send another message by creating a new
         * URLConnection
         */            
    }
}
